const path = require('path');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const HtmlBeautifyPlugin = require('html-beautify-webpack-plugin');
const shorthash = Math.random().toString(36).slice(-5);

module.exports = (env) => {
  // webpack --env.dev => isDev: true
  const isDev = env ? env.dev : false;
  console.log(`🔧 isDev: ${isDev}`);

  return {
    mode: isDev ? 'development' : 'production',
    entry: [
      path.join(__dirname, 'src/style.scss'),
      path.join(__dirname, 'src/index.js'),
    ],
    output: {
      path: path.join(__dirname, 'dist'),
      filename: `bundle.${shorthash}.js`
    },
    devServer: {
      port: 3000,
      contentBase: path.resolve(__dirname, 'src/'),
      watchContentBase: true
    },
    module: {
      rules: [
        {
          test: /\.js$/,
          exclude: /node_modules/,
          use: {
            loader: 'babel-loader',
            options: {
              presets: [
                ['env', {'modules': false}]
              ]
            }
          }
        },
        {
          test: /\.js$/,
          exclude: /node_modules/,
          use: ['eslint-loader']
        },
        {
          test: /\.scss$/,
          use: ExtractTextPlugin.extract({
            use: [
              {
                loader: 'css-loader',
                options: {
                  url: false,
                  sourceMap: isDev,
                  minimize: !isDev,
                  importLoaders: 2
                }
              },
              {
                loader: 'postcss-loader',
                options: {
                  sourceMap: isDev,
                  plugins: [require('autoprefixer')]
                },
              },
              {
                loader: 'sass-loader',
                options: {
                  sourceMap: isDev,
                  outputStyle: 'expanded',
                }
              },
              'csscomb-loader',
            ]
          })
        },
      ]
    },
    plugins: [
      new ExtractTextPlugin(`style.${shorthash}.css`),
      new HtmlWebpackPlugin({
        template: './src/index.html',
        copyright: '(c) iwb.jp',
      }),
      new CleanWebpackPlugin(['dist/*.*']),
      new HtmlBeautifyPlugin({
        config: {
          html: {
            end_with_newline: true,
            indent_size: 2,
            indent_inner_html: true,
            unformatted: ['p', 'i', 'b', 'span']
          }
        },
        replace: ['type="text/javascript"']
      }),
    ],
    // JSが複数ある場合はsplitChunksを使用
    // optimization: {
    //   splitChunks: {
    //     cacheGroups: {
    //       commons: {
    //         name: 'commons',
    //         chunks: 'initial'
    //       }
    //     }
    //   }
    // }
  }
};
